@@ -141,7 +141,7 @@ struct bluetooth_data {
 	pthread_t hw_thread;				/* Makes virtual hw pointer move */
 	int pipefd[2];					/* Inter thread communication */
 	int stopped;
-	sig_atomic_t reset;             /* Request XRUN handling */
+	sig_atomic_t reset;				/* Request XRUN handling */
 };
 
 static int audioservice_send(int sk, const bt_audio_msg_header_t *msg);
@@ -332,10 +332,10 @@ static int bluetooth_prepare(snd_pcm_ioplug_t *io)
 {
 	struct bluetooth_data *data = io->private_data;
 	char c = 'w';
-	char buf[BT_AUDIO_IPC_PACKET_SIZE];
-	struct bt_streamstart_req *start_req = (void*) buf;
-	bt_audio_rsp_msg_header_t *rsp_hdr = (void*) buf;
-	struct bt_streamfd_ind *streamfd_ind = (void*) buf;
+	char buf[BT_SUGGESTED_BUFFER_SIZE];
+	struct bt_start_stream_req *req = (void *) buf;
+	struct bt_start_stream_rsp *rsp = (void *) buf;
+	struct bt_new_stream_ind *ind = (void *) buf;
 	uint32_t period_count = io->buffer_size / io->period_size;
 	int opt_name, err;
 	struct timeval t = { 0, period_count };
@@ -363,27 +363,24 @@ static int bluetooth_prepare(snd_pcm_ioplug_t *io)
 		data->hw_ptr = io->period_size;
 
 	/* send start */
-	memset(start_req, 0, BT_AUDIO_IPC_PACKET_SIZE);
-	start_req->h.msg_type = BT_STREAMSTART_REQ;
+	memset(req, 0, BT_SUGGESTED_BUFFER_SIZE);
+	req->h.type = BT_REQUEST;
+	req->h.name = BT_START_STREAM;
+	req->h.length = sizeof(*req);
 
-	err = audioservice_send(data->server.fd, &start_req->h);
+	err = audioservice_send(data->server.fd, &req->h);
 	if (err < 0)
 		return err;
 
-	err = audioservice_expect(data->server.fd, &rsp_hdr->msg_h,
-					BT_STREAMSTART_RSP);
+	rsp->h.length = sizeof(*rsp);
+	err = audioservice_expect(data->server.fd, &rsp->h,
+					BT_START_STREAM);
 	if (err < 0)
 		return err;
 
-	if (rsp_hdr->posix_errno != 0) {
-		SNDERR("BT_START failed : %s(%d)",
-					strerror(rsp_hdr->posix_errno),
-					rsp_hdr->posix_errno);
-		return -rsp_hdr->posix_errno;
-	}
-
-	err = audioservice_expect(data->server.fd, &streamfd_ind->h,
-					BT_STREAMFD_IND);
+	ind->h.length = sizeof(*ind);
+	err = audioservice_expect(data->server.fd, &ind->h,
+					BT_NEW_STREAM);
 	if (err < 0)
 		return err;
 
@@ -432,41 +429,39 @@ static int bluetooth_hsp_hw_params(snd_pcm_ioplug_t *io,
 					snd_pcm_hw_params_t *params)
 {
 	struct bluetooth_data *data = io->private_data;
-	char buf[BT_AUDIO_IPC_PACKET_SIZE];
-	bt_audio_rsp_msg_header_t *rsp_hdr = (void*) buf;
-	struct bt_setconfiguration_req *setconf_req = (void*) buf;
-	struct bt_setconfiguration_rsp *setconf_rsp = (void*) buf;
+	char buf[BT_SUGGESTED_BUFFER_SIZE];
+	struct bt_set_configuration_req *req = (void *) buf;
+	struct bt_set_configuration_rsp *rsp = (void *) buf;
 	int err;
 
 	DBG("Preparing with io->period_size=%lu io->buffer_size=%lu",
 					io->period_size, io->buffer_size);
 
-	memset(setconf_req, 0, BT_AUDIO_IPC_PACKET_SIZE);
-	setconf_req->h.msg_type = BT_SETCONFIGURATION_REQ;
-	strncpy(setconf_req->device, data->alsa_config.device, 18);
-	setconf_req->transport = BT_CAPABILITIES_TRANSPORT_SCO;
-	setconf_req->access_mode = (io->stream == SND_PCM_STREAM_PLAYBACK ?
+	memset(req, 0, BT_SUGGESTED_BUFFER_SIZE);
+	req->h.type = BT_REQUEST;
+	req->h.name = BT_SET_CONFIGURATION;
+	req->h.length = sizeof(*req);
+
+	strncpy(req->device, data->alsa_config.device, 18);
+	req->codec.transport = BT_CAPABILITIES_TRANSPORT_SCO;
+	req->codec.length = sizeof(pcm_capabilities_t);
+	req->access_mode = (io->stream == SND_PCM_STREAM_PLAYBACK ?
 			BT_CAPABILITIES_ACCESS_MODE_WRITE :
 			BT_CAPABILITIES_ACCESS_MODE_READ);
 
-	err = audioservice_send(data->server.fd, &setconf_req->h);
+	req->h.length += req->codec.length - sizeof(req->codec);
+	err = audioservice_send(data->server.fd, &req->h);
 	if (err < 0)
 		return err;
 
-	err = audioservice_expect(data->server.fd, &rsp_hdr->msg_h,
-					BT_SETCONFIGURATION_RSP);
+	rsp->h.length = sizeof(*rsp);
+	err = audioservice_expect(data->server.fd, &rsp->h,
+					BT_SET_CONFIGURATION);
 	if (err < 0)
 		return err;
 
-	if (rsp_hdr->posix_errno != 0) {
-		SNDERR("BT_SETCONFIGURATION failed : %s(%d)",
-					strerror(rsp_hdr->posix_errno),
-					rsp_hdr->posix_errno);
-		return -rsp_hdr->posix_errno;
-	}
-
-	data->transport = setconf_rsp->transport;
-	data->link_mtu = setconf_rsp->link_mtu;
+	data->transport = rsp->transport;
+	data->link_mtu = rsp->link_mtu;
 
 	return 0;
 }
@@ -675,10 +670,9 @@ static int bluetooth_a2dp_hw_params(snd_pcm_ioplug_t *io,
 {
 	struct bluetooth_data *data = io->private_data;
 	struct bluetooth_a2dp *a2dp = &data->a2dp;
-	char buf[BT_AUDIO_IPC_PACKET_SIZE];
-	bt_audio_rsp_msg_header_t *rsp_hdr = (void*) buf;
-	struct bt_setconfiguration_req *setconf_req = (void*) buf;
-	struct bt_setconfiguration_rsp *setconf_rsp = (void*) buf;
+	char buf[BT_SUGGESTED_BUFFER_SIZE];
+	struct bt_set_configuration_req *req = (void *) buf;
+	struct bt_set_configuration_rsp *rsp = (void *) buf;
 	int err;
 
 	DBG("Preparing with io->period_size=%lu io->buffer_size=%lu",
@@ -688,33 +682,34 @@ static int bluetooth_a2dp_hw_params(snd_pcm_ioplug_t *io,
 	if (err < 0)
 		return err;
 
-	memset(setconf_req, 0, BT_AUDIO_IPC_PACKET_SIZE);
-	setconf_req->h.msg_type = BT_SETCONFIGURATION_REQ;
-	strncpy(setconf_req->device, data->alsa_config.device, 18);
-	setconf_req->transport = BT_CAPABILITIES_TRANSPORT_A2DP;
-	setconf_req->sbc_capabilities = a2dp->sbc_capabilities;
-	setconf_req->access_mode = (io->stream == SND_PCM_STREAM_PLAYBACK ?
+	memset(req, 0, BT_SUGGESTED_BUFFER_SIZE);
+	req->h.type = BT_REQUEST;
+	req->h.name = BT_SET_CONFIGURATION;
+	req->h.length = sizeof(*req);
+
+	strncpy(req->device, data->alsa_config.device, 18);
+	memcpy(&req->codec, &a2dp->sbc_capabilities,
+			sizeof(a2dp->sbc_capabilities));
+
+	req->codec.transport = BT_CAPABILITIES_TRANSPORT_A2DP;
+	req->codec.length = sizeof(a2dp->sbc_capabilities);
+	req->access_mode = (io->stream == SND_PCM_STREAM_PLAYBACK ?
 			BT_CAPABILITIES_ACCESS_MODE_WRITE :
 			BT_CAPABILITIES_ACCESS_MODE_READ);
+	req->h.length += req->codec.length - sizeof(req->codec);
 
-	err = audioservice_send(data->server.fd, &setconf_req->h);
+	err = audioservice_send(data->server.fd, &req->h);
 	if (err < 0)
 		return err;
 
-	err = audioservice_expect(data->server.fd, &rsp_hdr->msg_h,
-					BT_SETCONFIGURATION_RSP);
+	rsp->h.length = sizeof(*rsp);
+	err = audioservice_expect(data->server.fd, &rsp->h,
+					BT_SET_CONFIGURATION);
 	if (err < 0)
 		return err;
 
-	if (rsp_hdr->posix_errno != 0) {
-		SNDERR("BT_SETCONFIGURATION failed : %s(%d)",
-					strerror(rsp_hdr->posix_errno),
-					rsp_hdr->posix_errno);
-		return -rsp_hdr->posix_errno;
-	}
-
-	data->transport = setconf_rsp->transport;
-	data->link_mtu = setconf_rsp->link_mtu;
+	data->transport = rsp->transport;
+	data->link_mtu = rsp->link_mtu;
 
 	/* Setup SBC encoder now we agree on parameters */
 	bluetooth_a2dp_setup(a2dp);
@@ -1083,7 +1078,7 @@ static snd_pcm_sframes_t bluetooth_a2dp_write(snd_pcm_ioplug_t *io,
 		/* No space left for another frame then send */
 		if (a2dp->count + written >= data->link_mtu) {
 			avdtp_write(data);
-			printf("sending packet %d, count %d, link_mtu %u",
+			DBG("sending packet %d, count %d, link_mtu %u",
 						a2dp->seq_num, a2dp->count,
 							data->link_mtu);
 		}
@@ -1288,8 +1283,9 @@ static int bluetooth_a2dp_hw_constraint(snd_pcm_ioplug_t *io)
 
 	/* supported buffer sizes
 	 * (can be used as 3*8192, 6*4096, 12*2048, ...) */
-	err = snd_pcm_ioplug_set_param_minmax(io, SND_PCM_IOPLUG_HW_BUFFER_BYTES,
-					      8192*3, 8192*3);
+	err = snd_pcm_ioplug_set_param_minmax(io,
+						SND_PCM_IOPLUG_HW_BUFFER_BYTES,
+						8192*3, 8192*3);
 	if (err < 0)
 		return err;
 
@@ -1495,9 +1491,12 @@ static int bluetooth_parse_config(snd_config_t *conf,
 static int audioservice_send(int sk, const bt_audio_msg_header_t *msg)
 {
 	int err;
+	uint16_t length;
+
+	length = msg->length ? msg->length : BT_SUGGESTED_BUFFER_SIZE;
 
 	DBG("sending %s", bt_audio_strmsg(msg->msg_type));
-	if (send(sk, msg, BT_AUDIO_IPC_PACKET_SIZE, 0) > 0)
+	if (send(sk, msg, length, 0) > 0)
 		err = 0;
 	else {
 		err = -errno;
@@ -1511,19 +1510,23 @@ static int audioservice_send(int sk, const bt_audio_msg_header_t *msg)
 static int audioservice_recv(int sk, bt_audio_msg_header_t *inmsg)
 {
 	int err;
-	const char *type;
+	const char *type, *name;
+	uint16_t length;
+
+	length = inmsg->length ? inmsg->length : BT_SUGGESTED_BUFFER_SIZE;
 
 	DBG("trying to receive msg from audio service...");
-	if (recv(sk, inmsg, BT_AUDIO_IPC_PACKET_SIZE, 0) > 0) {
-		type = bt_audio_strmsg(inmsg->msg_type);
-		if (type) {
-			DBG("Received %s", type);
+	if (recv(sk, inmsg, length, 0) > 0) {
+		type = bt_audio_strtype(inmsg->type);
+		name = bt_audio_strname(inmsg->name);
+		if (type && name) {
+			DBG("Received %s - %s", type, name);
 			err = 0;
 		} else {
 			err = -EINVAL;
-			SNDERR("Bogus message type %d "
+			SNDERR("Bogus message type %d - name %d"
 					"received from audio service",
-					inmsg->msg_type);
+					inmsg->type, inmsg->name);
 		}
 	} else {
 		err = -errno;
@@ -1534,31 +1537,70 @@ static int audioservice_recv(int sk, bt_audio_msg_header_t *inmsg)
 	return err;
 }
 
-static int audioservice_expect(int sk, bt_audio_msg_header_t *rsp_hdr,
-				int expected_type)
+static int audioservice_expect(int sk, bt_audio_msg_header_t *rsp,
+				int expected_name)
 {
-	int err = audioservice_recv(sk, rsp_hdr);
-	if (err == 0) {
-		if (rsp_hdr->msg_type != expected_type) {
-			err = -EINVAL;
-			SNDERR("Bogus message %s received while "
-					"%s was expected",
-					bt_audio_strmsg(rsp_hdr->msg_type),
-					bt_audio_strmsg(expected_type));
-		}
+	bt_audio_error_t *error;
+	int err = audioservice_recv(sk, rsp);
+
+	if (err != 0)
+		return err;
+
+	if (rsp->name != expected_name) {
+		err = -EINVAL;
+		SNDERR("Bogus message %s received while %s was expected",
+				bt_audio_strname(rsp->name),
+				bt_audio_strname(expected_name));
 	}
+
+	if (rsp->type == BT_ERROR) {
+		error = (void *) rsp;
+		SNDERR("%s failed : %s(%d)",
+					bt_audio_strname(rsp->name),
+					strerror(error->posix_errno),
+					error->posix_errno);
+		return -error->posix_errno;
+	}
+
 	return err;
 }
 
+static int bluetooth_parse_capabilities(struct bluetooth_data *data,
+					struct bt_get_capabilities_rsp *rsp)
+{
+	int bytes_left = rsp->h.length - sizeof(*rsp);
+	codec_capabilities_t *codec = (void *) rsp->data;
+
+	data->transport = codec->transport;
+
+	if (codec->transport != BT_CAPABILITIES_TRANSPORT_A2DP)
+		return 0;
+
+	while (bytes_left > 0) {
+		if (codec->type == BT_A2DP_CODEC_SBC)
+			break;
+
+		bytes_left -= codec->length;
+		codec = (void *) codec + codec->length;
+	}
+
+	if (bytes_left <= 0 ||
+			codec->length != sizeof(data->a2dp.sbc_capabilities))
+		return -EINVAL;
+
+	memcpy(&data->a2dp.sbc_capabilities, codec, codec->length);
+
+	return 0;
+}
+
 static int bluetooth_init(struct bluetooth_data *data, snd_pcm_stream_t stream,
 				snd_config_t *conf)
 {
 	int sk, err;
 	struct bluetooth_alsa_config *alsa_conf = &data->alsa_config;
-	char buf[BT_AUDIO_IPC_PACKET_SIZE];
-	bt_audio_rsp_msg_header_t *rsp_hdr = (void*) buf;
-	struct bt_getcapabilities_req *getcaps_req = (void*) buf;
-	struct bt_getcapabilities_rsp *getcaps_rsp = (void*) buf;
+	char buf[BT_SUGGESTED_BUFFER_SIZE];
+	struct bt_get_capabilities_req *req = (void *) buf;
+	struct bt_get_capabilities_rsp *rsp = (void *) buf;
 
 	memset(data, 0, sizeof(struct bluetooth_data));
 
@@ -1594,36 +1636,30 @@ static int bluetooth_init(struct bluetooth_data *data, snd_pcm_stream_t stream,
 		goto failed;
 	}
 
-	memset(getcaps_req, 0, BT_AUDIO_IPC_PACKET_SIZE);
-	getcaps_req->h.msg_type = BT_GETCAPABILITIES_REQ;
-	getcaps_req->flags = 0;
+	memset(req, 0, BT_SUGGESTED_BUFFER_SIZE);
+	req->h.type = BT_REQUEST;
+	req->h.name = BT_GET_CAPABILITIES;
+	req->h.length = sizeof(*req);
+
 	if (alsa_conf->autoconnect)
-		getcaps_req->flags |= BT_FLAG_AUTOCONNECT;
-	strncpy(getcaps_req->device, alsa_conf->device, 18);
+		req->flags |= BT_FLAG_AUTOCONNECT;
+	strncpy(req->device, alsa_conf->device, 18);
 	if (alsa_conf->has_transport)
-		getcaps_req->transport = alsa_conf->transport;
+		req->transport = alsa_conf->transport;
 	else
-		getcaps_req->transport = BT_CAPABILITIES_TRANSPORT_ANY;
+		req->transport = BT_CAPABILITIES_TRANSPORT_ANY;
 
-	err = audioservice_send(data->server.fd, &getcaps_req->h);
+	err = audioservice_send(data->server.fd, &req->h);
 	if (err < 0)
 		goto failed;
 
-	err = audioservice_expect(data->server.fd, &rsp_hdr->msg_h, BT_GETCAPABILITIES_RSP);
+	rsp->h.length = 0;
+	err = audioservice_expect(data->server.fd, &rsp->h,
+					BT_GET_CAPABILITIES);
 	if (err < 0)
 		goto failed;
 
-	if (rsp_hdr->posix_errno != 0) {
-		SNDERR("BT_GETCAPABILITIES failed : %s(%d)",
-					strerror(rsp_hdr->posix_errno),
-					rsp_hdr->posix_errno);
-		return -rsp_hdr->posix_errno;
-	}
-
-	data->transport = getcaps_rsp->transport;
-
-	if (getcaps_rsp->transport == BT_CAPABILITIES_TRANSPORT_A2DP)
-		data->a2dp.sbc_capabilities = getcaps_rsp->sbc_capabilities;
+	bluetooth_parse_capabilities(data, rsp);
 
 	return 0;
 
