@@ -43,6 +43,7 @@
 
 #include <gdbus.h>
 
+#include "dbus-common.h"
 #include "logging.h"
 #include "adapter.h"
 #include "error.h"
@@ -264,7 +265,42 @@ static DBusMessage *list_adapters(DBusConnection *conn,
 	return reply;
 }
 
+static DBusMessage *get_properties(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	DBusMessage *reply;
+	DBusMessageIter iter;
+	DBusMessageIter dict;
+	GSList *list;
+	char **array;
+	int i;
+
+	reply = dbus_message_new_method_return(msg);
+	if (!reply)
+		return NULL;
+
+	dbus_message_iter_init_append(reply, &iter);
+
+	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
+			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
+			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
+			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
+
+	array = g_new0(char *, g_slist_length(adapters) + 1);
+	for (i = 0, list = adapters; list; list = list->next, i++) {
+		struct btd_adapter *adapter = list->data;
+		array[i] = (char *) adapter_get_path(adapter);
+	}
+	dict_append_array(&dict, "Adapters", DBUS_TYPE_OBJECT_PATH, &array, i);
+	g_free(array);
+
+	dbus_message_iter_close_container(&iter, &dict);
+
+	return reply;
+}
+
 static GDBusMethodTable manager_methods[] = {
+	{ "GetProperties",	"",	"a{sv}",get_properties	},
 	{ "DefaultAdapter",	"",	"o",	default_adapter	},
 	{ "FindAdapter",	"s",	"o",	find_adapter	},
 	{ "ListAdapters",	"",	"ao",	list_adapters	},
@@ -272,6 +308,7 @@ static GDBusMethodTable manager_methods[] = {
 };
 
 static GDBusSignalTable manager_signals[] = {
+	{ "PropertyChanged",		"sv"	},
 	{ "AdapterAdded",		"o"	},
 	{ "AdapterRemoved",		"o"	},
 	{ "DefaultAdapterChanged",	"o"	},
@@ -366,23 +403,43 @@ GSList *manager_get_adapters(void)
 	return adapters;
 }
 
+static void manager_update_adapters(void)
+{
+	GSList *list;
+	char **array;
+	int i;
+
+	array = g_new0(char *, g_slist_length(adapters) + 1);
+	for (i = 0, list = adapters; list; list = list->next, i++) {
+		struct btd_adapter *adapter = list->data;
+		array[i] = (char *) adapter_get_path(adapter);
+	}
+
+	emit_array_property_changed(connection, "/",
+					MANAGER_INTERFACE, "Adapters",
+					DBUS_TYPE_OBJECT_PATH, &array);
+
+	g_free(array);
+}
+
 static void manager_add_adapter(struct btd_adapter *adapter)
 {
 	const gchar *path = adapter_get_path(adapter);
 
+	adapters = g_slist_append(adapters, adapter);
+
 	g_dbus_emit_signal(connection, "/",
 			MANAGER_INTERFACE, "AdapterAdded",
 			DBUS_TYPE_OBJECT_PATH, &path,
 			DBUS_TYPE_INVALID);
 
-	adapters = g_slist_append(adapters, adapter);
-
 	if (default_adapter_id < 0) {
 		int new_default = hci_get_route(NULL);
 
-		if (new_default >= 0)
-			manager_set_default_adapter(new_default);
+		manager_set_default_adapter(new_default);
 	}
+
+	manager_update_adapters();
 }
 
 static void manager_remove_adapter(struct btd_adapter *adapter)
@@ -390,6 +447,8 @@ static void manager_remove_adapter(struct btd_adapter *adapter)
 	uint16_t dev_id = adapter_get_dev_id(adapter);
 	const gchar *path = adapter_get_path(adapter);
 
+	manager_update_adapters();
+
 	g_dbus_emit_signal(connection, "/",
 			MANAGER_INTERFACE, "AdapterRemoved",
 			DBUS_TYPE_OBJECT_PATH, &path,
@@ -398,8 +457,7 @@ static void manager_remove_adapter(struct btd_adapter *adapter)
 	if (default_adapter_id == dev_id || default_adapter_id < 0) {
 		int new_default = hci_get_route(NULL);
 
-		if (new_default >= 0)
-			manager_set_default_adapter(new_default);
+		manager_set_default_adapter(new_default);
 	}
 
 	adapters = g_slist_remove(adapters, adapter);
@@ -478,11 +536,17 @@ int manager_get_default_adapter()
 
 void manager_set_default_adapter(int id)
 {
-	struct btd_adapter *adapter = manager_find_adapter_by_id(id);
-	const gchar *path = adapter_get_path(adapter);
+	struct btd_adapter *adapter;
+	const gchar *path;
 
 	default_adapter_id = id;
 
+	adapter = manager_find_adapter_by_id(id);
+	if (!adapter)
+		return;
+
+	path = adapter_get_path(adapter);
+
 	g_dbus_emit_signal(connection, "/",
 			MANAGER_INTERFACE,
 			"DefaultAdapterChanged",
