@@ -46,6 +46,7 @@
 
 #include <dbus/dbus.h>
 
+#include "hcid.h"
 #include "logging.h"
 #include "textfile.h"
 
@@ -366,7 +367,7 @@ static void link_key_notify(int dev, bdaddr_t *sba, void *ptr)
 						DISCONNECT_CP_SIZE, &cp);
 		}
 	} else
-		hcid_dbus_bonding_process_complete(sba, dba, 0);
+		hcid_dbus_link_key_notify(sba, dba);
 
 	io_data[dev_id].pin_length = -1;
 }
@@ -774,8 +775,7 @@ static inline void auth_complete(int dev, bdaddr_t *sba, void *ptr)
 	if (get_bdaddr(dev, sba, btohs(evt->handle), &dba) < 0)
 		return;
 
-	if (evt->status)
-		hcid_dbus_bonding_process_complete(sba, &dba, evt->status);
+	hcid_dbus_bonding_process_complete(sba, &dba, evt->status);
 }
 
 static inline void conn_request(int dev, bdaddr_t *sba, void *ptr)
@@ -789,6 +789,20 @@ static inline void conn_request(int dev, bdaddr_t *sba, void *ptr)
 	write_remote_class(sba, &evt->bdaddr, class);
 }
 
+static void delete_channel(GIOChannel *chan)
+{
+	int i;
+
+	/* Look for the GIOChannel in the table */
+	for (i = 0; i < HCI_MAX_DEV; i++)
+		if (io_data[i].channel == chan) {
+			stop_security_manager(i);
+			return;
+		}
+
+	error("IO channel not found in the io_data table");
+}
+
 static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer data)
 {
 	unsigned char buf[HCI_MAX_EVENT_SIZE], *ptr = buf;
@@ -799,14 +813,14 @@ static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer
 	GIOError err;
 
 	if (cond & (G_IO_NVAL | G_IO_HUP | G_IO_ERR)) {
-		g_io_channel_unref(chan);
+		delete_channel(chan);
 		return FALSE;
 	}
 
 	if ((err = g_io_channel_read(chan, (gchar *) buf, sizeof(buf), &len))) {
 		if (err == G_IO_ERROR_AGAIN)
 			return TRUE;
-		g_io_channel_unref(chan);
+		delete_channel(chan);
 		return FALSE;
 	}
 
